#!/usr/bin/env bash

# Переключаем раскладку
    xdotool key --delay 20 Mode_switch
# Делаем бэкап последней записи буфера
    BACKUPCLIPB=$(xsel -b)
    if [[ -n $1 ]]; then
	xdotool key --delay 150 Control
	xdotool key --delay 150 Control+Left
#   	xdotool keydown --delay 150 Control key --delay 150 Left keyup --delay 150 Control
	xdotool key --delay 50  Shift+End
    else
	xdotool key --delay 150 End
	xdotool key --delay 150 End
	xdotool key --delay 50 Shift+Home
    fi
    xdotool key --delay 50 Control+Insert
    SELCLIPB=$(xsel -b)
    CONVCLIPB=$(echo -n "${SELCLIPB}" | sed "y/\
abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ[]{};':\",.\/<>?@#\$^&\`~фисвуапршолдьтщзйкыегмцчняФИСВУАПРШОЛДЬТЩЗЙКЫЕГМЦЧНЯхъХЪжэЖЭбюБЮ№ёЁ/\
фисвуапршолдьтщзйкыегмцчняФИСВУАПРШОЛДЬТЩЗЙКЫЕГМЦЧНЯхъХЪжэЖЭбю.БЮ,\"№;:?ёЁabcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ[]{};':\",.<>#\`~/")
# Конвертируем символы
    echo -n "${CONVCLIPB}" | xsel -b -i; sleep 0.2
    xdotool key --delay 50 Shift_R+Insert
# Восстанавливаем бэкап последней записи буфера
    echo -n "${BACKUPCLIPB}" | xsel -b -i
#   echo -n "${BACKUPCLIPB}" | xsel -p -i




